#include <stdio.h>
#include <stdint.h>
#include <inttypes.h>

#define DEBUG 0

void ext_eucl (int a, int b, int *ss, int *tt, int *ggt)
{
  int r0 = a;
  int r1 = b;
  int s0 = 1;
  int s1 = 0;
  int t0 = 0;
  int t1 = 1;
  int r2 = r0 % r1;
  while (r2)
    {
      int q = r0 / r1;
      int s2 = s0 - q * s1;
      int t2 = t0 - q * t1;
      r0 = r1;
      s0 = s1;
      t0 = t1;
      r1 = r2;
      s1 = s2;
      t1 = t2;
      r2 = r0 % r1;
    }
  *ggt = r1;
  *ss = s1;
  *tt = t1;
}

int gen_key (int e, int n)
{
  int d;
  int t;
  int ggt;
  ext_eucl (e, n, &d, &t, &ggt);
  while (d < 0)
    d += n;
  if (ggt != 1)
    return -1;
  else
    return d;
}

int mod_power (uint64_t b, int e, uint64_t N)
{
  uint64_t result = 1;
  for (int i = 0; i < e; i++)
    {
      result = (result * b) % N;
      if (DEBUG > 1)
        printf ("i = %d, result = %" PRIu64 "\n", i, result);
    }
  return result;
}

int encrypt (int m, int e, int N)
{
  return mod_power (m, e, N);
}

int decrypt (int c, int d, int N)
{
  return mod_power (c, d, N);
}

int main (void)
{
  int p = 11;
  int q = 13;
  int N = p * q;
  int n = (p - 1) * (q - 1);
  int e = 23;
  printf ("Öffentlicher Schlüssel: e = %d, N = %d\n", e, N);
  int d = gen_key (e, n);
  printf ("Geheimer Schlüssel: d = %d, n = %d", d, n);
  if (DEBUG > 0)
    printf (", d · e mod n = %d", (d * e) % n);
  printf ("\n");
  if (DEBUG > 0)
    printf ("2^24 = %d\n", mod_power (2, 24, 2000000000));
  int m = 7;
  int c = encrypt (m, e, N); 
  printf ("Nachricht: %d\nVerschlüsselte Nachricht: %d\n", m, c);
  int m2 = decrypt (c, d, N);
  printf ("Entschlüsselte Nachricht: %d\n", m2);
  return 0;
}
